@@ -280,6 +280,9 @@ sub learn {
     return;
   }
 
+  my $learner_said_ham_hits = -1.0;
+  my $learner_said_spam_hits = 1.0;
+
   if ($isspam) {
     my $required_body_hits = 3;
     my $required_head_hits = 3;
@@ -294,6 +297,18 @@ sub learn {
 		  $self->{head_only_hits}." < ".$required_head_hits.")");
       return;
     }
+    if ($self->{learned_hits} < $learner_said_ham_hits) {
+      dbg ("auto-learn? no: learner indicated ham (".
+		  $self->{learned_hits}." < ".$learner_said_ham_hits.")");
+      return;
+    }
+
+  } else {
+    if ($self->{learned_hits} > $learner_said_spam_hits) {
+      dbg ("auto-learn? no: learner indicated spam (".
+		  $self->{learned_hits}." > ".$learner_said_spam_hits.")");
+      return;
+    }
   }
 
   dbg ("auto-learn? yes, ".($isspam?"spam ($hits > $max)":"ham ($hits < $min)"));
@@ -1301,6 +1316,7 @@ sub get {
       $_ = join ("\n", grep { defined($_) && length($_) > 0 }
 		$self->{msg}->get_header ('X-Message-Id'),
 		$self->{msg}->get_header ('Resent-Message-Id'),
+		$self->{msg}->get_header ('X-Original-Message-ID'), # bug 2122
 		$self->{msg}->get_header ('Message-Id'));
     }
     # a conventional header
